@@ -6,14 +6,17 @@ from ipaddress import IPv4Address
 
 import sentry_sdk
 import yaml
+from cached_property import cached_property
 from pydantic import AnyHttpUrl
 from pydantic import BaseModel
 from pydantic import BaseSettings
 from pydantic import Extra
+from pydantic import conint
 from pydantic import validator
 from sentry_sdk.integrations.logging import LoggingIntegration
 
 from . import __version__
+from . import utils
 from .dict_merge import dict_merge
 from .logging import setup_logging
 
@@ -30,6 +33,7 @@ __all__ = (
     'ServerModel',
     'Settings',
     'ConfigurationError',
+    'settings',
 )
 
 logger = logging.getLogger('sqproxy.config')
@@ -41,18 +45,37 @@ class ConfigurationError(Exception):
 
 class NetworkModel(BaseModel):
     server_ip: IPv4Address
-    server_port: int
-    bind_ip: IPv4Address
-    bind_port: int
+    server_port: conint(ge=1, le=65535)
+    bind_ip: typing.Optional[IPv4Address] = None
+    bind_port: typing.Optional[conint(ge=0, le=65535)] = 0
     ebpf_no_redirect: bool = False
 
     class Config:
         extra = Extra.forbid
 
+    @validator('bind_ip', always=True)
+    def _set_default_bind_ip(cls, bind_ip, values):
+        if not bind_ip:
+            bind_ip = values['server_ip']
+
+        return bind_ip
+
+    @validator('bind_port', always=True)
+    def _set_default_bind_port(cls, bind_port, values):
+        if bind_port is None or bind_port == 0:
+            server_port = values['server_port']
+
+            if utils.is_port_available(server_port + 800):
+                # try to use pretty port
+                bind_port = server_port + 800
+            else:
+                bind_port = utils.get_available_port()
+
+        return bind_port
+
 
 class EntrypointModel(BaseModel):
-    """Allow override proxy server entrypoint
-    """
+    """Allow override proxy server entrypoint"""
 
     path: str  #: path to callable which should return :class:`QueryProxy`-based object
     obj: typing.Optional[typing.Callable[..., QueryProxyType]] = None
@@ -116,6 +139,9 @@ class EBPFModel(BaseModel):
         extra = Extra.forbid
 
 
+NamedServersType = typing.List[typing.Tuple[str, ServerModel]]
+
+
 class Settings(BaseSettings):
     sentry_dsn: typing.Optional[AnyHttpUrl] = None
     confdir_0: pathlib.Path = '/etc/sqproxy/conf.d/'
@@ -127,8 +153,10 @@ class Settings(BaseSettings):
     class Config:
         env_file = '.env'
         env_prefix = 'SQPROXY_'
+        keep_untouched = (cached_property,)
 
-    def get_merged_config_data(self):
+    @cached_property
+    def merged_config_data(self):
         return load_configs(iter_config_files(self.confdir_0, self.confdir_1))
 
     @validator('loglevel')
@@ -138,14 +166,30 @@ class Settings(BaseSettings):
         _checkLevel(v)
         return v
 
+    @cached_property
+    def servers(self) -> NamedServersType:
+        merged_config_data = self.merged_config_data
+
+        servers = merged_config_data.get('servers')
+        if servers:
+            servers = [(name, ServerModel.parse_obj(server)) for name, server in servers.items()]
+
+        return servers or []
+
+    @cached_property
+    def ebpf(self) -> typing.Optional[EBPFModel]:
+        ebpf = self.merged_config_data.get('ebpf')
+        if not ebpf:
+            return None
+        return EBPFModel.parse_obj(ebpf)
+
 
 def _apply_defaults(target, defaults):
     target.update(dict_merge(defaults, target))
 
 
-def _get_config(data: dict, global_defaults) -> typing.Tuple[typing.Dict, typing.Dict]:
-    """Get full and ready to use config
-    """
+def _get_old_config(data: dict, global_defaults) -> typing.Tuple[typing.Dict, typing.Dict]:
+    """Get full and ready to use config"""
     defaults = data.pop('defaults', None) or {}
 
     servers_data = data.get('servers')
@@ -162,6 +206,31 @@ def _get_config(data: dict, global_defaults) -> typing.Tuple[typing.Dict, typing
     return data, defaults
 
 
+def _get_config(data: dict, global_defaults) -> typing.Tuple[typing.Dict, typing.Dict]:
+    """Get full and ready to use config"""
+    if 'global' in (data.get('defaults') or {}):
+        data, defaults = _get_old_config(data, global_defaults)
+        is_global = defaults.pop('global')
+        defaults = defaults['values']
+        defaults['__global__'] = is_global
+        return data, defaults
+
+    defaults = data.pop('defaults', None) or {}
+
+    servers_data = data.get('servers')
+    if not servers_data:
+        return data, defaults
+
+    for server in servers_data.values():
+        for g_defaults in global_defaults:
+            _apply_defaults(server, g_defaults)
+
+        if defaults:
+            _apply_defaults(server, defaults)
+
+    return data, defaults
+
+
 def iter_config_files(*confdirs):
     for confdir in confdirs:
         if not confdir.exists():
@@ -195,7 +264,7 @@ def load_configs(paths: typing.Iterable[pathlib.Path]):
                     raise ConfigurationError('eBPF already configured')
                 ebpf_configured = True
 
-            if config_defaults.get('global', False):
+            if config_defaults.pop('__global__', False):
                 global_defaults.append(config_defaults)
 
             if config:
@@ -212,9 +281,9 @@ def load_configs(paths: typing.Iterable[pathlib.Path]):
     return whole_config
 
 
-def setup(settings_: Settings = None):
+def setup(settings_: Settings = None, reread: bool = False):
     global settings
-    if settings_ is None:
+    if settings_ is None or reread:
         logger.info('Re-read settings')
         settings_ = Settings()
 
@@ -226,7 +295,9 @@ def setup(settings_: Settings = None):
             event_level=logging.ERROR,  # Send errors as events
         )
         sentry_sdk.init(
-            dsn=settings.sentry_dsn, integrations=[sentry_logging], release=__version__,
+            dsn=settings.sentry_dsn,
+            integrations=[sentry_logging],
+            release=__version__,
         )
 
     setup_logging(loglevel=settings.loglevel, error_filename=settings.error_log.as_posix())
@@ -236,22 +307,18 @@ def setup(settings_: Settings = None):
     else:
         logger.debug('Sentry disabled')
 
-    merged_config_data = settings.get_merged_config_data()
-
-    servers_ = merged_config_data.get('servers')
-    if servers_:
-        global servers
-        servers = [(name, ServerModel.parse_obj(server)) for name, server in servers_.items()]
-
-    ebpf_ = merged_config_data.get('ebpf')
-    if ebpf_:
-        global ebpf
-        ebpf = EBPFModel.parse_obj(ebpf_)
+    return settings
 
 
 settings = Settings()
-servers = None  # type: typing.Optional[typing.List[typing.Tuple[str, ServerModel]]]
-ebpf = None  # type: typing.Optional[EBPFModel]
-
 
 setup(settings)
+
+
+def __getattr__(name):
+    if name == 'ebpf':
+        return settings.ebpf
+    elif name == 'servers':
+        return settings.servers
+    else:
+        raise AttributeError(name)
